@@ -17,8 +17,20 @@ from .annotation_helpers import (
 )
 from .api import API as _API
 from .common import image_path_to_annotation_paths
-# from .consensus_benchmark.benchmark import benchmark
-# from .consensus_benchmark.consensus import consensus
+
+try:
+    from .consensus_benchmark.benchmark import benchmark
+    from .consensus_benchmark.consensus import consensus
+except:
+    _WARNING_SHAPELY = "To use superannotate.benchmark or superannotate.consensus functions please install shapely package in Anaconda enviornment with # conda install shapely"
+
+    def benchmark(*args, **kwargs):
+        raise RuntimeError(_WARNING_SHAPELY)
+
+    def consensus(*args, **kwargs):
+        raise RuntimeError(_WARNING_SHAPELY)
+
+
 from .dataframe_filtering import (
     filter_annotation_instances, filter_images_by_comments,
     filter_images_by_tags
@@ -74,9 +86,11 @@ from .exceptions import (
     SANonExistingProjectNameException
 )
 from .input_converters.conversion import (
-    coco_split_dataset, convert_platform, convert_project_type,
-    export_annotation, import_annotation
+    coco_split_dataset, convert_project_type, export_annotation,
+    import_annotation, convert_json_version
 )
+
+from .old_to_new_format_convertor import update_json_format
 from .version import __version__
 
 formatter = logging.Formatter(fmt='SA-PYTHON-SDK - %(levelname)s - %(message)s')
