@@ -296,12 +296,12 @@ def download_annotation_classes_json(project, folder):
 
 
 def fill_class_and_attribute_names(annotations_json, annotation_classes_dict):
-    for r in annotations_json:
+    for r in annotations_json["instances"]:
         if "classId" in r and r["classId"] in annotation_classes_dict:
             r["className"] = annotation_classes_dict[r["classId"]]["name"]
             if "attributes" in r:
                 for attribute in r["attributes"]:
-                    if "groupId" in attribute and "id" in "attribute":
+                    if "groupId" in attribute and "id" in attribute:
                         attribute["groupName"] = annotation_classes_dict[
                             r["classId"]]["attribute_groups"][
                                 attribute["groupId"]]["name"]
@@ -311,10 +311,9 @@ def fill_class_and_attribute_names(annotations_json, annotation_classes_dict):
 
 
 def fill_class_and_attribute_ids(annotation_json, annotation_classes_dict):
-    for ann in annotation_json:
-        if (
-            "userId" in ann and "type" in ann and ann["type"] == "meta"
-        ) or "className" not in ann:
+    for ann in annotation_json["instances"]:
+        if "className" not in ann:
+            logger.warning("No className in annotation instance")
             continue
         annotation_class_name = ann["className"]
         if not annotation_class_name in annotation_classes_dict:
@@ -349,6 +348,15 @@ def fill_class_and_attribute_ids(annotation_json, annotation_classes_dict):
                 del attribute["groupId"]
 
 
+def check_annotation_json(annotation_json):
+    if "metadata" not in annotation_json or "width" not in annotation_json[
+        "metadata"] or "height" not in annotation_json[
+            "metadata"] or "name" not in annotation_json["metadata"]:
+        return False
+    else:
+        return True
+
+
 def get_annotation_classes_id_to_name(annotation_classes):
     annotation_classes_dict = {}
     for annotation_class in annotation_classes:
