@@ -753,13 +753,17 @@ def get_image_annotations(project, image_name, project_type=None):
             raise SABaseException(response.status_code, response.text)
         res_json = response.json()
         fill_class_and_attribute_names(res_json, annotation_classes_dict)
-        url = res["pixelSave"]["url"]
-        annotation_mask_filename = url.rsplit('/', 1)[-1]
-        headers = res["pixelSave"]["headers"]
-        response = requests.get(url=url, headers=headers)
-        if not response.ok:
-            raise SABaseException(response.status_code, response.text)
-        mask = io.BytesIO(response.content)
+        if len(res_json["instances"]) != 0:
+            url = res["pixelSave"]["url"]
+            annotation_mask_filename = url.rsplit('/', 1)[-1]
+            headers = res["pixelSave"]["headers"]
+            response = requests.get(url=url, headers=headers)
+            if not response.ok:
+                raise SABaseException(response.status_code, response.text)
+            mask = io.BytesIO(response.content)
+        else:
+            mask = None
+            annotation_mask_filename = None
         return {
             "annotation_json": res_json,
             "annotation_json_filename": annotation_json_filename,
@@ -800,11 +804,14 @@ def download_image_annotations(project, image_name, local_dir_path):
     else:
         with open(json_path, "w") as f:
             json.dump(annotation["annotation_json"], f, indent=4)
-        mask_path = Path(local_dir_path
-                        ) / annotation["annotation_mask_filename"]
+        if annotation["annotation_mask_filename"] is not None:
+            mask_path = Path(local_dir_path
+                            ) / annotation["annotation_mask_filename"]
+            with open(mask_path, "wb") as f:
+                f.write(annotation["annotation_mask"].getbuffer())
+        else:
+            mask_path = None
         return_filepaths.append(str(mask_path))
-        with open(mask_path, "wb") as f:
-            f.write(annotation["annotation_mask"].getbuffer())
 
     return tuple(return_filepaths)
 
@@ -861,7 +868,12 @@ def upload_image_annotations(
     :type mask: BytesIO or Pathlike (str or Path)
     """
 
-    if not isinstance(annotation_json, list):
+    if isinstance(annotation_json, list):
+        raise SABaseException(
+            0,
+            "Annotation JSON should be a dict object. You are using list object. If this is an old annotation format you can convert it to new format with superannotate.update_json_format SDK function"
+        )
+    if not isinstance(annotation_json, dict):
         if verbose:
             logger.info("Uploading annotations from %s.", annotation_json)
         annotation_json = json.load(open(annotation_json))
@@ -975,7 +987,7 @@ def create_fuse_image(
         if output_overlay:
             fi_pil_ovl = Image.fromarray(fi_ovl)
             draw_ovl = ImageDraw.Draw(fi_pil_ovl)
-        for annotation in annotation_json:
+        for annotation in annotation_json["instances"]:
             if "className" not in annotation:
                 continue
             color = class_color_dict[annotation["className"]]
@@ -1075,7 +1087,7 @@ def create_fuse_image(
     else:
         annotation_mask = np.array(Image.open(annotation_path[1]))
         # print(annotation_mask.shape, annotation_mask.dtype)
-        for annotation in annotation_json:
+        for annotation in annotation_json["instances"]:
             if "className" not in annotation or "parts" not in annotation:
                 continue
             color = class_color_dict[annotation["className"]]
