@@ -6,16 +6,17 @@ from pathlib import Path
 
 import boto3
 
+from .. import common
 from ..api import API
-from ..common import annotation_status_int_to_str, annotation_status_str_to_int
-from ..exceptions import SABaseException
+from ..exceptions import SABaseException, SAImageSizeTooLarge
 from .images import (
     delete_image, get_image_annotations, get_image_bytes, get_image_metadata,
-    set_image_annotation_status, upload_annotations_from_json_to_image
+    search_images, set_image_annotation_status, upload_image_annotations
 )
+from .project_api import get_project_metadata_bare
 from .projects import (
-    __create_image, _get_project_default_image_quality_in_editor,
-    get_image_array_to_upload, get_project_metadata
+    __create_image, get_project_default_image_quality_in_editor,
+    get_image_array_to_upload, upload_image_array_to_s3
 )
 
 logger = logging.getLogger("superannotate-python-sdk")
@@ -37,18 +38,22 @@ def upload_image_to_project(
     :type project: str or dict
     :param img: image to upload
     :type img: io.BytesIO() or Pathlike (str or Path)
+    :param image_name: image name to set on platform. If None and img is filepath,
+                       image name will be set to filename of the path
+    :type image_name: str
     :param annotation_status: value to set the annotation statuses of the uploaded image NotStarted InProgress QualityCheck Returned Completed Skipped
     :type annotation_status: str
     :param from_s3_bucket: AWS S3 bucket to use. If None then folder_path is in local filesystem
     :type from_s3_bucket: str
-    :param image_quality_in_editor: image quality (in percents) that will be seen in SuperAnnotate web annotation editor. If None default value will be used.
-    :type image_quality_in_editor: int
+    :param image_quality_in_editor: image quality be seen in SuperAnnotate web annotation editor.
+           Can be either "compressed" or "original".  If None then the default value in project settings will be used.
+    :type image_quality_in_editor: str
     """
     if not isinstance(project, dict):
-        project = get_project_metadata(project)
-    annotation_status = annotation_status_str_to_int(annotation_status)
+        project = get_project_metadata_bare(project)
+    annotation_status = common.annotation_status_str_to_int(annotation_status)
     if image_quality_in_editor is None:
-        image_quality_in_editor = _get_project_default_image_quality_in_editor(
+        image_quality_in_editor = get_project_default_image_quality_in_editor(
             project
         )
 
@@ -59,11 +64,20 @@ def upload_image_to_project(
             from_session = boto3.Session()
             from_s3 = from_session.resource('s3')
             from_s3_object = from_s3.Object(from_s3_bucket, img)
+            file_size = from_s3_object.content_length
+            if file_size > common.MAX_IMAGE_SIZE:
+                raise SAImageSizeTooLarge(file_size, img)
             img = io.BytesIO()
             from_s3_object.download_fileobj(img)
         else:
+            file_size = Path(img).stat().st_size
+            if file_size > common.MAX_IMAGE_SIZE:
+                raise SAImageSizeTooLarge(file_size, img)
             with open(img, "rb") as f:
                 img = io.BytesIO(f.read())
+    elif img.getbuffer().nbytes > common.MAX_IMAGE_SIZE:
+        raise SAImageSizeTooLarge(file_size)
+
     if image_name is not None:
         img_name = image_name
 
@@ -95,15 +109,12 @@ def upload_image_to_project(
     )
     s3_resource = s3_session.resource('s3')
     bucket = s3_resource.Bucket(res["bucket"])
-    orig_image, lores_image, huge_image, thumbnail_image = get_image_array_to_upload(
-        img, project["type"], image_quality_in_editor
-    )
     key = prefix + f'{img_name}'
     try:
-        bucket.put_object(Body=orig_image, Key=key)
-        bucket.put_object(Body=lores_image, Key=key + '___lores.jpg')
-        bucket.put_object(Body=huge_image, Key=key + '___huge.jpg')
-        bucket.put_object(Body=thumbnail_image, Key=key + '___thumb.jpg')
+        images_array = get_image_array_to_upload(
+            img, image_quality_in_editor, project["type"]
+        )
+        upload_image_array_to_s3(bucket, *images_array, key, project["type"])
     except Exception as e:
         raise SABaseException(0, "Couldn't upload to data server. " + e)
 
@@ -123,7 +134,8 @@ def copy_image(
     image_name,
     destination_project,
     include_annotations=False,
-    copy_annotation_status=False
+    copy_annotation_status=False,
+    copy_pin=False
 ):
     """Copy image to a project. The image's project is the same as destination
     project then the name will be changed to <image_name>_(<num>).<image_ext>,
@@ -139,11 +151,13 @@ def copy_image(
     :type include_annotations: bool
     :param copy_annotation_status: enables annotations status copy
     :type copy_annotation_status: bool
+    :param copy_pin: enables image pin status copy
+    :type copy_pin: bool
     """
     if not isinstance(source_project, dict):
-        source_project = get_project_metadata(source_project)
+        source_project = get_project_metadata_bare(source_project)
     if not isinstance(destination_project, dict):
-        destination_project = get_project_metadata(destination_project)
+        destination_project = get_project_metadata_bare(destination_project)
     img_b = get_image_bytes(source_project, image_name)
     img_metadata = get_image_metadata(source_project, image_name)
     new_name = image_name
@@ -172,21 +186,22 @@ def copy_image(
         annotations = get_image_annotations(source_project, image_name)
         if annotations["annotation_json"] is not None:
             if "annotation_mask" in annotations:
-                upload_annotations_from_json_to_image(
+                upload_image_annotations(
                     destination_project, new_name,
                     annotations["annotation_json"],
                     annotations["annotation_mask"]
                 )
             else:
-                upload_annotations_from_json_to_image(
+                upload_image_annotations(
                     destination_project, new_name,
                     annotations["annotation_json"]
                 )
     if copy_annotation_status:
         set_image_annotation_status(
-            destination_project, new_name,
-            annotation_status_int_to_str(img_metadata["annotation_status"])
+            destination_project, new_name, img_metadata["annotation_status"]
         )
+    if copy_pin:
+        pin_image(destination_project, new_name, img_metadata["is_pinned"])
 
     logger.info(
         "Copied image %s/%s to %s/%s.", source_project["name"], image_name,
@@ -198,8 +213,9 @@ def move_image(
     source_project,
     image_name,
     destination_project,
-    include_annotations=False,
-    copy_annotation_status=False
+    include_annotations=True,
+    copy_annotation_status=True,
+    copy_pin=True
 ):
     """Move image from source_project to destination_project. source_project
     and destination_project cannot be the same.
@@ -214,18 +230,99 @@ def move_image(
     :type include_annotations: bool
     :param copy_annotation_status: enables annotations status copy
     :type copy_annotation_status: bool
+    :param copy_pin: enables image pin status copy
+    :type copy_pin: bool
     """
     if not isinstance(source_project, dict):
-        source_project = get_project_metadata(source_project)
+        source_project = get_project_metadata_bare(source_project)
     if not isinstance(destination_project, dict):
-        destination_project = get_project_metadata(destination_project)
+        destination_project = get_project_metadata_bare(destination_project)
     if source_project == destination_project:
         raise SABaseException(
             0, "Cannot move image if source_project == destination_project."
         )
     copy_image(
         source_project, image_name, destination_project, include_annotations,
-        copy_annotation_status
+        copy_annotation_status, copy_pin
     )
     delete_image(source_project, image_name)
     logger.info("Deleted image %s/%s.", source_project["name"], image_name)
+
+
+def pin_image(project, image_name, pin=True):
+    """Pins (or unpins) image
+
+    :param project: project name or metadata of the project
+    :type project: str or dict
+    :param image_name: image name
+    :type image: str
+    :param pin: sets to pin if True, else unpins image
+    :type pin: bool
+    """
+    if not isinstance(project, dict):
+        project = get_project_metadata_bare(project)
+    img_metadata = get_image_metadata(project, image_name)
+    team_id, project_id, image_id = project["team_id"], project[
+        "id"], img_metadata["id"]
+    params = {"team_id": team_id, "project_id": project_id}
+    json_req = {"is_pinned": int(pin)}
+    response = _api.send_request(
+        req_type='PUT',
+        path=f'/image/{image_id}',
+        params=params,
+        json_req=json_req
+    )
+    if not response.ok:
+        raise SABaseException(
+            response.status_code, "Couldn't pin image " + response.text
+        )
+
+
+def assign_images(project, image_names, user):
+    """Assigns images to a user. The assignment role, QA or Annotator, will
+    be deduced from the user's role in the project. With SDK, the user can be
+    assigned to a role in the project with the share_project function.
+
+    :param project: project name or metadata of the project
+    :type project: str or dict
+    :param image_names: list of image names to assign
+    :type image_names: list of str
+    :param user: user email
+    :type user: str
+    """
+    logger.info("Assign %s images to user %s", len(image_names), user)
+    if len(image_names) == 0:
+        return
+    if not isinstance(project, dict):
+        project = get_project_metadata_bare(project)
+    folder_id = None
+    images = search_images(project, return_metadata=True)
+    image_dict = {}
+    for image in images:
+        image_dict[image["name"]] = image["id"]
+        if folder_id is None:
+            folder_id = image["folder_id"]
+        elif folder_id != image["folder_id"]:
+            raise SABaseException(0, "Folders not implemented yet")
+
+    image_ids = []
+    for image_name in image_names:
+        image_ids.append(image_dict[image_name])
+    team_id, project_id = project["team_id"], project["id"]
+    params = {
+        "team_id": team_id,
+        "project_id": project_id,
+        "folder_id": folder_id
+    }
+    json_req = {"user_id": user, "image_ids": image_ids}
+    response = _api.send_request(
+        req_type='POST',
+        path='/images/assign',
+        params=params,
+        json_req=json_req
+    )
+    if not response.ok:
+        raise SABaseException(
+            response.status_code, "Couldn't assign images " + response.text
+        )
+    # print(response.json())
