@@ -8,11 +8,9 @@ from ..exceptions import SABaseException
 from .export_from_sa_conversions import export_from_sa
 from .import_to_sa_conversions import import_to_sa
 from .sa_conversion import (
-    sa_convert_platform, sa_convert_project_type, split_coco
+    degrade_json, sa_convert_project_type, split_coco, upgrade_json
 )
 
-AVAILABLE_PLATFORMS = ["Desktop", "Web"]
-
 ALLOWED_TASK_TYPES = [
     'panoptic_segmentation', 'instance_segmentation', 'keypoint_detection',
     'object_detection', 'vector_annotation'
@@ -114,18 +112,6 @@ def _passes_sanity_checks(args):
             0, "Please enter valid task '%s'" % (ALLOWED_TASK_TYPES)
         )
 
-    if 'platform' in args:
-        if args.platform not in AVAILABLE_PLATFORMS:
-            raise SABaseException(
-                0, "Please enter valid platform: 'Desktop' or 'Web'"
-            )
-
-    if args.project_type == "Pixel" and args.platform == "Desktop":
-        raise SABaseException(
-            0,
-            "Sorry, but Desktop Application doesn't support 'Pixel' projects."
-        )
-
 
 def _passes_converter_sanity(args, direction):
     converter_values = (args.project_type, args.task)
@@ -153,7 +139,6 @@ def export_annotation(
     dataset_name,
     project_type="Vector",
     task="object_detection",
-    platform="Web",
 ):
     """Converts SuperAnnotate annotation formate to the other annotation formats. Currently available (project_type, task) combinations for converter
     presented below:
@@ -171,9 +156,9 @@ def export_annotation(
     ==============  ======================
 
     :param input_dir: Path to the dataset folder that you want to convert.
-    :type input_dir: str
+    :type input_dir: Pathlike(str or Path)
     :param output_dir: Path to the folder, where you want to have converted dataset.
-    :type output_dir: str
+    :type output_dir: Pathlike(str or Path)
     :param dataset_format: One of the formats that are possible to convert. Available candidates are: ["COCO"]
     :type dataset_format: str
     :param dataset_name: Will be used to create json file in the output_dir.
@@ -189,8 +174,6 @@ def export_annotation(
                  'instance_segmentation' 'Pixel' project_type converts instance masks and 'Vector' project_type generates bounding boxes and polygons from instance masks. Masks should be in the input folder if it is 'Pixel' project_type.
                  'object_detection' converts objects from/to available annotation format
     :type task: str
-    :param platform: SuperAnnotate has both 'Web' and 'Desktop' platforms. Choose from which one you are converting. (Default: "Web")
-    :type platform: str
 
     """
 
@@ -206,7 +189,6 @@ def export_annotation(
         dataset_name=dataset_name,
         project_type=project_type,
         task=task,
-        platform=platform,
     )
 
     _passes_sanity_checks(args)
@@ -222,7 +204,6 @@ def import_annotation(
     dataset_name='',
     project_type="Vector",
     task="object_detection",
-    platform="Web",
     images_root='',
     images_extensions=['jpg']
 ):
@@ -259,6 +240,7 @@ def import_annotation(
     Vector          object_detection
     Vector          instance_segmentation
     Vector          vector_annotation
+    Pixel           instance_segmentation
     ==============  ======================
 
     ==============  ======================
@@ -329,9 +311,9 @@ def import_annotation(
     ==============  ======================
 
     :param input_dir: Path to the dataset folder that you want to convert.
-    :type input_dir: str
+    :type input_dir: Pathlike(str or Path)
     :param output_dir: Path to the folder, where you want to have converted dataset.
-    :type output_dir: str
+    :type output_dir: Pathlike(str or Path)
     :param dataset_format: Annotation format to convert SuperAnnotate annotation format. Available candidates are: ["COCO", "VOC", "LabelBox", "DataLoop",
                         "Supervisely", 'VGG', 'YOLO', 'SageMake', 'VoTT', 'GoogleCloud']
     :type dataset_format: str
@@ -348,14 +330,10 @@ def import_annotation(
                  'instance_segmentation' 'Pixel' project_type converts instance masks and 'Vector' project_type generates bounding boxes and polygons from instance masks. Masks should be in the input folder if it is 'Pixel' project_type.
                  'object_detection' converts objects from/to available annotation format
                  'vector_annotation' can be used to convert all annotations (point, ellipse, circule, cuboid and etc) to SuperAnnotate vector project.
-    :param platform: SuperAnnotate has both 'Web' and 'Desktop' platforms. Choose to which platform you want convert. (Default: "Web")
-    :type platform: str
     :param images_root: Additonal path to images directory in input_dir
-    :type platform: str
-    :param images_root: Additonal path to images directory in input_dir
-    :type platform: str
+    :type images_root: str
     :param image_extensions: List of image files xtensions in the images_root folder
-    :type platform: list
+    :type image_extensions: list
 
     """
 
@@ -371,7 +349,6 @@ def import_annotation(
         dataset_name=dataset_name,
         project_type=project_type,
         task=task,
-        platform=platform,
         images_root=images_root,
         images_extensions=images_extensions
     )
@@ -382,45 +359,13 @@ def import_annotation(
     import_to_sa(args)
 
 
-def convert_platform(input_dir, output_dir, input_platform):
-    """ Converts SuperAnnotate input file structure from one platform too another.
-
-    :param input_dir: Path to the dataset folder that you want to convert.
-    :type input_dir: str or PathLike
-    :param output_dir: Path to the folder where you want to have converted files.
-    :type output_dir: str or PathLike
-    :param input_platform: Original platform format type
-    :type input_platform: str
-
-    """
-    param_info = [
-        (input_dir, 'input_dir', (str, Path)),
-        (output_dir, 'output_dir', (str, Path)),
-        (input_platform, 'input_platform', str),
-    ]
-    for param in param_info:
-        _type_sanity(param[0], param[1], param[2])
-
-    if input_platform not in AVAILABLE_PLATFORMS:
-        raise SABaseException(
-            0, "Please enter valid platform: 'Desktop' or 'Web'"
-        )
-
-    if isinstance(input_dir, str):
-        input_dir = Path(input_dir)
-    if isinstance(output_dir, str):
-        output_dir = Path(output_dir)
-
-    sa_convert_platform(input_dir, output_dir, input_platform)
-
-
 def convert_project_type(input_dir, output_dir):
     """ Converts SuperAnnotate 'Vector' project type to 'Pixel' or reverse.
 
     :param input_dir: Path to the dataset folder that you want to convert.
-    :type input_dir: str or PathLike
+    :type input_dir: Pathlike(str or Path)
     :param output_dir: Path to the folder where you want to have converted files.
-    :type output_dir: str or PathLike
+    :type output_dir: Pathlike(str or Path)
 
     """
     param_info = [
@@ -444,11 +389,11 @@ def coco_split_dataset(
     """ Splits COCO dataset to few datsets.
 
     :param coco_json_path: Path to main COCO JSON dataset, which should be splitted.
-    :type coco_json_path: str or PathLike
+    :type coco_json_path: Pathlike(str or Path)
     :param image_dir: Path to all images in the original dataset.
-    :type coco_json_path: str or PathLike
+    :type coco_json_path: str or Pathlike
     :param coco_json_path: Path to the folder where you want to output splitted COCO JSON files.
-    :type coco_json_path: str or PathLike
+    :type coco_json_path: str or Pathlike
     :param dataset_list_name: List of dataset names.
     :type dataset_list_name: list
     :param ratio_list: List of ratios for each splitted dataset.
@@ -505,3 +450,40 @@ def _type_sanity(var, var_name, var_type):
                 var_name, var_type, type(var)
             )
         )
+
+
+def convert_json_version(input_dir, output_dir, version=2):
+    """
+    Converts SuperAnnotate JSON versions. Newest JSON version is 2.
+
+    :param input_dir: Path to the dataset folder that you want to convert.
+    :type input_dir: Pathlike(str or Path)
+    :param output_dir: Path to the folder, where you want to have converted dataset.
+    :type output_dir: Pathlike(str or Path)
+    :param version: Output version number. Currently is either 1 or 2. Default value is 2. It will upgrade version 1 to  version 2. Set 1 to degrade from version 2 to version 1.
+    :type version: int
+
+    :return: List of converted files
+    :rtype: list
+    """
+    param_info = [
+        (input_dir, 'input_dir', (str, Path)),
+        (output_dir, 'output_dir', (str, Path)), (version, 'version', int)
+    ]
+    for param in param_info:
+        _type_sanity(param[0], param[1], param[2])
+
+    if isinstance(input_dir, str):
+        input_dir = Path(input_dir)
+    if isinstance(output_dir, str):
+        output_dir = Path(output_dir)
+    output_dir.mkdir(parents=True, exist_ok=True)
+
+    if version == 2:
+        converted_files = upgrade_json(input_dir, output_dir)
+    elif version == 1:
+        converted_files = degrade_json(input_dir, output_dir)
+    else:
+        raise SABaseException(0, "'version' is either 1 or 2.")
+
+    return converted_files
