@@ -1,17 +1,13 @@
 import json
+from pathlib import Path
 
 import cv2
 import numpy as np
-
-from pathlib import Path
-from collections import defaultdict
 from tqdm import tqdm
 
-import time
-
-from .coco_api import (_maskfrRLE, decode)
-
-from ....common import id2rgb
+from ....common import write_to_json
+from ..sa_json_helper import _create_sa_json, _create_vector_instance
+from .coco_api import _maskfrRLE, decode
 
 
 def annot_to_polygon(annot):
@@ -32,139 +28,98 @@ def annot_to_polygon(annot):
     return segments
 
 
-def coco_instance_segmentation_to_sa_vector(coco_path, images_path):
-    coco_json = json.load(open(coco_path))
-    image_id_to_annotations = {}
-    cat_id_to_cat = {}
-    for cat in coco_json['categories']:
-        cat_id_to_cat[cat['id']] = cat
-
-    instance_groups = {}
-    for annot in coco_json['annotations']:
-        if 'id' in annot:
-            if annot['id'] in instance_groups:
-                instance_groups[annot['id']] += 1
-            else:
-                instance_groups[annot['id']] = 1
-
-    for annot in tqdm(coco_json['annotations'], "Converting annotations"):
-        if isinstance(annot['segmentation'], dict):
-            annot['segmentation'] = annot_to_polygon(annot['segmentation'])
-
-        cat = cat_id_to_cat[annot['category_id']]
-        sa_polygon_loader = []
-        groupid = 0
-        for polygon in annot['segmentation']:
-            if 'id' in annot:
-                if annot['id'] in instance_groups and instance_groups[
-                    annot['id']] > 1:
-                    groupid = annot['id']
-            sa_obj = {
-                'type': 'polygon',
-                'points': polygon,
-                'className': cat['name'],
-                'classId': cat['id'],
-                'attributes': [],
-                'probability': 100,
-                'locked': False,
-                'visible': True,
-                'imageId': annot['image_id']
-            }
-            sa_polygon_loader.append(sa_obj)
-
-        for polygon in sa_polygon_loader:
-            if polygon['imageId'] not in image_id_to_annotations:
-                image_id_to_annotations[polygon['imageId']] = [polygon]
-            else:
-                image_id_to_annotations[polygon['imageId']].append(polygon)
-
-    sa_jsons = {}
+def save_sa_jsons(coco_json, img_id_to_annot, output_dir):
     for img in tqdm(coco_json['images'], "Writing annotations to disk"):
         if 'file_name' in img:
             image_path = Path(img['file_name']).name
         else:
             image_path = img['coco_url'].split('/')[-1]
 
-        if img['id'] not in image_id_to_annotations:
-            f_loader = []
+        if str(img['id']) not in img_id_to_annot:
+            sa_instances = []
         else:
-            f_loader = image_id_to_annotations[img['id']]
-        file_name = image_path + "___objects.json"
-        sa_jsons[file_name] = f_loader
-    return sa_jsons
+            sa_instances = img_id_to_annot[str(img['id'])]
+        file_name = "%s___objects.json" % image_path
 
-
-def coco_object_detection_to_sa_vector(coco_path, images_path):
-    coco_json = json.load(open(coco_path))
-    image_id_to_annotations = {}
-    cat_id_to_cat = {}
-    for cat in coco_json['categories']:
-        cat_id_to_cat[cat['id']] = cat
-
-    instance_groups = {}
-    for annot in coco_json['annotations']:
-        if 'id' in annot:
-            if annot['id'] in instance_groups:
-                instance_groups[annot['id']] += 1
-            else:
-                instance_groups[annot['id']] = 1
-
-    for annot in tqdm(coco_json['annotations'], "Converting annotations"):
-        if isinstance(annot['segmentation'], dict):
-            annot['segmentation'] = annot_to_polygon(annot['segmentation'])
-
-        cat = cat_id_to_cat[annot['category_id']]
-        groupid = 0
-        for polygon in annot['segmentation']:
-            if 'id' in annot:
-                if annot['id'] in instance_groups and instance_groups[
-                    annot['id']] > 1:
-                    groupid = annot['id']
-
-        sa_dict_bbox = {
-            'type': 'bbox',
-            'points':
-                {
-                    'x1': annot['bbox'][0],
-                    'y1': annot['bbox'][1],
-                    'x2': annot['bbox'][0] + annot['bbox'][2],
-                    'y2': annot['bbox'][1] + annot['bbox'][3]
-                },
-            'className': cat['name'],
-            'classId': cat['id'],
-            'attributes': [],
-            'probability': 100,
-            'locked': False,
-            'visible': True,
-            # 'groupId': groupid,
-            'imageId': annot['image_id']
+        sa_metadata = {
+            'name': image_path,
+            'width': img['width'],
+            'height': img['height']
         }
-
-        if sa_dict_bbox['imageId'] not in image_id_to_annotations:
-            image_id_to_annotations[sa_dict_bbox['imageId']] = [sa_dict_bbox]
-        else:
-            image_id_to_annotations[sa_dict_bbox['imageId']
-                                   ].append(sa_dict_bbox)
-
-    sa_jsons = {}
-    for img in tqdm(coco_json['images'], "Writing annotations to disk"):
-        if 'file_name' in img:
-            image_path = Path(img['file_name']).name
-        else:
-            image_path = img['coco_url'].split('/')[-1]
-
-        if img['id'] not in image_id_to_annotations:
-            f_loader = []
-        else:
-            f_loader = image_id_to_annotations[img['id']]
-        file_name = image_path + "___objects.json"
-        sa_jsons[file_name] = f_loader
-    return sa_jsons
+        json_template = _create_sa_json(sa_instances, sa_metadata)
+        write_to_json(output_dir / file_name, json_template)
 
 
-def coco_keypoint_detection_to_sa_vector(coco_path, images_path):
+def coco_instance_segmentation_to_sa_vector(coco_path, output_dir):
+    coco_json = json.load(open(coco_path))
+    cat_id_to_cat = {}
+    for cat in coco_json['categories']:
+        cat_id_to_cat[cat['id']] = cat
+
+    instance_groups = {}
+    for annot in coco_json['annotations']:
+        if 'id' in annot:
+            if annot['id'] in instance_groups:
+                instance_groups[annot['id']] += 1
+            else:
+                instance_groups[annot['id']] = 1
+
+    image_id_to_annotations = {}
+    for annot in tqdm(coco_json['annotations'], "Converting annotations"):
+        if isinstance(annot['segmentation'], dict):
+            annot['segmentation'] = annot_to_polygon(annot['segmentation'])
+
+        cat = cat_id_to_cat[annot['category_id']]
+        groupid = 0
+        for polygon in annot['segmentation']:
+            if 'id' in annot and annot[
+                'id'] in instance_groups and instance_groups[annot['id']] > 1:
+                groupid = annot['id']
+
+            sa_obj = _create_vector_instance(
+                'polygon', polygon, {}, [], cat['name']
+            )
+            if groupid != 0:
+                sa_obj['groupId'] = groupid
+
+            if str(annot['image_id']) not in image_id_to_annotations:
+                image_id_to_annotations[str(annot['image_id'])] = [sa_obj]
+            else:
+                image_id_to_annotations[str(annot['image_id'])].append(sa_obj)
+
+    save_sa_jsons(coco_json, image_id_to_annotations, output_dir)
+
+
+def coco_object_detection_to_sa_vector(coco_path, output_dir):
+    coco_json = json.load(open(coco_path))
+    cat_id_to_cat = {}
+    for cat in coco_json['categories']:
+        cat_id_to_cat[cat['id']] = cat
+
+    image_id_to_annotations = {}
+    for annot in tqdm(coco_json['annotations'], "Converting annotations"):
+        if isinstance(annot['segmentation'], dict):
+            annot['segmentation'] = annot_to_polygon(annot['segmentation'])
+        cat = cat_id_to_cat[annot['category_id']]
+
+        points = (
+            annot['bbox'][0], annot['bbox'][1],
+            annot['bbox'][0] + annot['bbox'][2],
+            annot['bbox'][1] + annot['bbox'][3]
+        )
+
+        sa_obj = _create_vector_instance('bbox', points, {}, [], cat['name'])
+
+        if str(annot['image_id']) not in image_id_to_annotations:
+            image_id_to_annotations[str(annot['image_id'])] = [sa_obj]
+        else:
+            image_id_to_annotations[str(annot['image_id'])].append(sa_obj)
+
+    save_sa_jsons(coco_json, image_id_to_annotations, output_dir)
+
+
+def coco_keypoint_detection_to_sa_vector(coco_path, output_dir):
     coco_json = json.load(open(coco_path))
-    loader = []
 
     cat_id_to_cat = {}
     for cat in coco_json["categories"]:
@@ -174,6 +129,7 @@ def coco_keypoint_detection_to_sa_vector(coco_path, images_path):
             "skeleton": cat["skeleton"]
         }
 
+    image_id_to_annotations = {}
     for annot in tqdm(coco_json['annotations'], 'Converting annotations'):
         if annot['num_keypoints'] > 0:
             sa_points = [
@@ -189,41 +145,19 @@ def coco_keypoint_detection_to_sa_vector(coco_path, images_path):
                 keypoint_names = cat_id_to_cat[annot["category_id"]
                                               ]['keypoints']
 
-                sa_template = {
-                    'type': 'template',
-                    'classId': annot['category_id'],
-                    'probability': 100,
-                    'points': [],
-                    'connections': [],
-                    'attributes': [],
-                    'attributeNames': [],
-                    # 'groupId': annot['id'],
-                    'pointLabels': {},
-                    'locked': False,
-                    'visible': True,
-                    'templateId': -1,
-                    'className': cat_id_to_cat[annot["category_id"]]["name"],
-                    'templateName': 'skeleton',
-                    'imageId': annot['image_id']
-                }
-
                 bad_points = []
                 id_mapping = {}
                 index = 1
+                points = []
                 for point_index, point in enumerate(sa_points):
                     if sa_points[point_index] == (0, 0):
                         bad_points.append(point_index + 1)
                         continue
                     id_mapping[point_index + 1] = index
-                    sa_template['points'].append(
-                        {
-                            'id': index,
-                            'x': point[0],
-                            'y': point[1]
-                        }
-                    )
+                    points.append({'id': index, 'x': point[0], 'y': point[1]})
                     index += 1
 
+                connections = []
                 for connection in cat_id_to_cat[annot["category_id"]
                                                ]['skeleton']:
 
@@ -236,7 +170,7 @@ def coco_keypoint_detection_to_sa_vector(coco_path, images_path):
                     if from_point in bad_points or to_point in bad_points:
                         continue
 
-                    sa_template['connections'].append(
+                    connections.append(
                         {
                             'id': index + 1,
                             'from': id_mapping[from_point],
@@ -244,22 +178,20 @@ def coco_keypoint_detection_to_sa_vector(coco_path, images_path):
                         }
                     )
 
+                pointLabels = {}
                 for kp_index, kp_name in enumerate(keypoint_names):
                     if kp_index + 1 in bad_points:
                         continue
-                    sa_template['pointLabels'][id_mapping[kp_index + 1] -
-                                               1] = kp_name
+                    pointLabels[id_mapping[kp_index + 1] - 1] = kp_name
 
-                for img in coco_json['images']:
-                    if sa_template['imageId'] == img['id']:
-                        loader.append((img['id'], sa_template))
+                sa_obj = _create_vector_instance(
+                    'template', points, pointLabels, [],
+                    cat_id_to_cat[annot["category_id"]]["name"], connections
+                )
+                if str(annot['image_id']) not in image_id_to_annotations:
+                    image_id_to_annotations[str(annot['image_id'])] = [sa_obj]
+                else:
+                    image_id_to_annotations[str(annot['image_id']
+                                               )].append(sa_obj)
 
-        sa_jsons = {}
-        for img in coco_json['images']:
-            f_loader = []
-            for img_id, img_data in loader:
-                if img['id'] == img_id:
-                    f_loader.append(img_data)
-            file_name = str(Path(img['file_name']).name) + "___objects.json"
-            sa_jsons[file_name] = f_loader
-    return sa_jsons
+    save_sa_jsons(coco_json, image_id_to_annotations, output_dir)
